#!/bin/sh

. /etc/init.d/rc.functions
. /etc/conf.d/rc.conf

get_settings4()
{
    unset ipaddr ipmask ipnet
    ipaddr=`get_var ${device} IP`
    ipmask=`get_var ${device} MASK`
    if [ -n "${ipaddr}" ] && [ -n "${ipmask}" ] ; then
        ipnet=`ipcalc -n ${ipaddr}/${ipmask} | sed -e 's|NETWORK=||'`/${ipmask}
    else
        ipnet=""
    fi
}

get_settings6()
{
    unset ip6addr ip6mask ip6net
    ip6addr=`get_var ${device} IP6`
    ip6mask=`get_var ${device} MASK6`
    if [ -n "${ip6addr}" ] && [ -n "${ip6mask}" ] ; then
        ip6net=""
        if [ "${GLOBAL_CFNET6}" == "yes" ] && [ -n "${GLOBAL_CFNET6_PREFIX}" ] ; then
            ip6net="${GLOBAL_CFNET6_PREFIX}::/48"
        fi
    else
        ip6net=""
    fi
}

get_tmp_devices()
{
    GLOBAL_DEVICES=""
    . ${tmpnetworkdir}/${host}.conf
    localdevices=0
    remotedevices=0 
    for device in ${GLOBAL_DEVICES} ; do
	get_settings4
        get_settings6
	if [ "${host}" == "${thishost}" ] ; then
	    let localdevices++
            eval localipnets$localdevices=${ipnet}
            eval localipaddrs$localdevices=${ipaddr}
            eval localip6nets$localdevices=${ip6net}
            eval localip6addrs$localdevices=${ip6addr}
	else
	    let remotedevices++
            eval remoteipnets$remotedevices=${ipnet}
            eval remoteipaddrs$remotedevices=${ipaddr}
            eval remoteip6nets$remotedevices=${ip6net}
            eval remoteip6addrs$remotedevices=${ip6addr}
	fi
    done
}

analysenet()
{
    unset localdevices localipnets localipaddrs localip6nets localip6addrs
    domain=`cat /etc/conf.d/domainname`
    thishost=`cat /etc/conf.d/hostname`
    gen_default_hosts ${tmphostsfile}

    host=${thishost}
    get_tmp_devices

    # for all localdevices calculate ip networks
    for ld in `seq ${localdevices}` ; do
        eval localipnet="\$localipnets$ld"
        eval localipaddr="\$localipaddrs$ld"
	if [ -n "$localipnet" ] && [ -n "$localipaddr" ] ; then
            echo "${localipaddr} ${thishost}.${domain} ${thishost}" >> ${tmphostsfile}
        fi
        eval localip6net="\$localip6nets$ld"
        eval localip6addr="\$localip6addrs$ld"
        if [ -n "$localip6net" ] && [ -n "$localip6addr" ] ; then
            echo "${localip6addr} ${thishost}.${domain} ${thishost}" >> ${tmphostsfile}
        fi
        if [ -n "$localip6net" ] || [ -n "$localipnet" ] ; then
	   for remote in ${tmpnetworkdir}/*.conf ; do
               host=${remote##*/}
	       host=${host%.conf}
	       if [ "$thishost" != "$host" ] ;	then
                   
                   # if a network is found calculate ip networks of all remotedevices and add entry if match
                   unset remotedevices remoteipnets remoteipaddrs remoteip6nets remoteip6addrs                
                   get_tmp_devices

	           for rd in `seq ${remotedevices}` ; do
                       eval remoteipnet="\$remoteipnets$rd"
                       eval remoteipaddr="\$remoteipaddrs$rd"
                       if [ -n "$remoteipnet" ] && [ -n "$remoteipaddr" ] && [ "$localipnet" == "$remoteipnet" ] ; then
	    		   echo "$remoteipaddr ${host}.${domain} ${host}" >> ${tmphostsfile}
		       fi
                       eval remoteip6net="\$remoteip6nets$rd"
                       eval remoteip6addr="\$remoteip6addrs$rd"
                       if [ -n "$remoteip6net" ] && [ -n "$remoteip6addr" ] && [ "$localip6net" == "$remoteip6net" ] ; then
                           echo "$remoteip6addr ${host}.${domain} ${host}" >> ${tmphostsfile}
                       fi
		   done

    		fi
	    done
    	fi
    done
}

case $1 in  
    temporary)
        logger "$0 $@"        
        tmpnetworkdir="/etc/network.test"
        tmphostsfile="/etc/hosts.tmp"        
        analysenet
        chmod 644 $tmphostsfile
        ;;
    permanent)
        logger "$0 $@"
        tmpnetworkdir="/etc/network.d"
        tmphostsfile="/etc/hosts"
        analysenet
        chmod 644 ${tmphostsfile}
        ;;
    *)
        echo "Usage: $0 temporary|permanent"
        exit 1
        ;;
esac

